/*
 * $Id: ndir.c,v 1.4 1995/11/08 15:09:29 miura Exp $
 *
 * ndir.c -- portable directory routines
 * Copyright (c) 1995, Shigeki Miura <miura@rpts.cl.nec.co.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#if defined (_MSC_VER) && (_MSC_VER >= 800)
#pragma warning (disable : 4057 4201 4214 4514)
#endif /* _MSC_VER */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <tchar.h>
#include <string.h>
#include <stdlib.h>
#include <ndir.h>

static char rcsid[] = "$Id: ndir.c,v 1.4 1995/11/08 15:09:29 miura Exp $";


DIR *
opendir (const _TCHAR *name)
{
  DIR *dirp;
  DWORD dwAttributes;

  if ((dwAttributes = GetFileAttributes (name)) == 0xffffffff
      || !(dwAttributes & FILE_ATTRIBUTE_DIRECTORY))
    return NULL;

  if ((dirp = malloc (sizeof (DIR))) == NULL)
    return NULL;

  _tcscpy (dirp->dd_name, name);
  dirp->dd_fd = NULL;
  dirp->dd_loc = 0;

  return dirp;
}


void
closedir (DIR *dirp)
{
  FindClose (dirp->dd_fd);
  free (dirp);
}


struct direct *
readdir (DIR *dirp)
{
  static struct direct dir;
  WIN32_FIND_DATA find_data;

  if (dirp->dd_fd == NULL)
    {
      _TCHAR szFilename[MAX_PATH];
      _TCHAR *p;

      GetFullPathName (dirp->dd_name, sizeof (szFilename) / sizeof (_TCHAR),
		       szFilename, NULL);
      if ((p = _tcsrchr (szFilename, _T('\\'))) != NULL
	  && (*_tcsinc (p) == _T('\0')))
	_tcscat (szFilename, _T("*"));
      else
	_tcscat (szFilename, _T("\\*"));
      dirp->dd_fd = FindFirstFile (szFilename, &find_data);
      if (dirp->dd_fd == INVALID_HANDLE_VALUE)
	  return NULL;
      dirp->dd_loc = 0;
    }
  else
    {
      if (!FindNextFile (dirp->dd_fd, &find_data))
	return NULL;
      dirp->dd_loc++;
    }

  dir.d_fileno = 0;
  dir.d_reclen = sizeof (struct direct);
  dir.d_namlen = (unsigned short) _tcslen (find_data.cFileName);
  _tcscpy (dir.d_name, find_data.cFileName);

  dir.d_attrib = find_data.dwFileAttributes;

  return &dir;
}


void
rewinddir (DIR *dirp)
{
  FindClose (dirp->dd_fd);
  dirp->dd_fd = NULL;
  dirp->dd_loc = 0;
}


long
telldir (DIR *dirp)
{
  return dirp->dd_loc;
}


void
seekdir (DIR *dirp, long pos)
{
  rewinddir (dirp);

  while (--pos > 0)
    readdir (dirp);
}


#ifdef TEST

#include <stdio.h>
#include <stdlib.h>
#include <locale.h>


int
_tmain (int argc, _TCHAR **argv)
{
  DIR *dir;
  struct direct *entry = NULL;
  _TCHAR *name = _T(".");
  _TCHAR szLang[256], *pszLocale;

  GetLocaleInfo (LOCALE_USER_DEFAULT, LOCALE_SENGLANGUAGE,
		 szLang, sizeof (szLang) / sizeof (_TCHAR));
  pszLocale = _tsetlocale (LC_ALL, szLang);

  if (argc > 1)
    name = argv[1];

  if ((dir = opendir (name)) == NULL)
    {
      _ftprintf (stderr, _T("can't open directory `%s'.\n"), name);
      return EXIT_FAILURE;
    }

  while ((entry = readdir (dir)) != NULL)
    _tprintf (_T("%3d: '%s'\n"), entry->d_namlen, entry->d_name);

  closedir (dir);

  _tprintf (_T("done.\n"));

  return EXIT_SUCCESS;
}

#endif /* TEST */
