/*
 * $Id: stat.c,v 1.4 1995/11/08 17:25:38 miura Exp $
 *
 * stat.c -- stat(), fstat() for Windows NT with NTFS
 * Copyright (c) 1995, Shigeki Miura <miura@rpts.cl.nec.co.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <io.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>


static char rcsid[] = "$Id: stat.c,v 1.4 1995/11/08 17:25:38 miura Exp $";


static time_t
convfiletime (FILETIME *ft)
{
  SYSTEMTIME st;
  FILETIME ftLocal;
  struct tm tm;

  FileTimeToLocalFileTime (ft, &ftLocal);
  FileTimeToSystemTime (&ftLocal, &st);
  tm.tm_year = st.wYear - 1900;
  tm.tm_mon = st.wMonth - 1;
  tm.tm_wday = st.wDayOfWeek;
  tm.tm_mday = st.wDay;
  tm.tm_hour = st.wHour;
  tm.tm_min = st.wMinute;
  tm.tm_sec = st.wSecond;
  tm.tm_yday = -1;
  tm.tm_isdst = -1;

  return mktime (&tm);
}


int
nt_stat (const char *name, struct stat *buf)
{
  int ret;
  HANDLE hFile;
  BY_HANDLE_FILE_INFORMATION fi;

  ret = stat (name, buf);

  if (!ret)
    {
      hFile = CreateFile (name, 0, 0, NULL, OPEN_EXISTING,
			  FILE_FLAG_BACKUP_SEMANTICS, 0);
      if (hFile != INVALID_HANDLE_VALUE)
	{
	  if (GetFileInformationByHandle (hFile, &fi))
	    {
#if 0
	      buf->st_dev = buf->st_rdev = fi.dwVolumeSerialNumber;
#endif
	      buf->st_ino = (_ino_t) fi.nFileIndexLow;
	      buf->st_nlink = (short) fi.nNumberOfLinks;
	      buf->st_size = fi.nFileSizeLow;
	      buf->st_atime = convfiletime (&fi.ftLastAccessTime);
	      buf->st_mtime = convfiletime (&fi.ftLastWriteTime);
	      buf->st_ctime = convfiletime (&fi.ftCreationTime);
	    }
	  CloseHandle (hFile);
	}
      if (buf->st_atime == 0xffffffff)
	buf->st_atime = 0;
      if (buf->st_mtime == 0xffffffff)
	buf->st_mtime = 0;
      if (buf->st_ctime == 0xffffffff)
	buf->st_ctime = 0;
    }

  return ret;
}


int
nt_fstat (int handle, struct stat *buf)
{
  int ret;
  BY_HANDLE_FILE_INFORMATION fi;

  ret = fstat (handle, buf);

  if (!ret)
    {
      if (GetFileInformationByHandle ((HANDLE) _get_osfhandle (handle), &fi))
	{
#if 0
	  buf->st_dev = buf->st_rdev = fi.dwVolumeSerialNumber;
#endif
	  buf->st_ino = (_ino_t) fi.nFileIndexLow;
	  buf->st_nlink = (short) fi.nNumberOfLinks;
	  buf->st_size = fi.nFileSizeLow;
	  buf->st_atime = convfiletime (&fi.ftLastAccessTime);
	  buf->st_mtime = convfiletime (&fi.ftLastWriteTime);
	  buf->st_ctime = convfiletime (&fi.ftCreationTime);
	}
      if (buf->st_atime == 0xffffffff)
	buf->st_atime = 0;
      if (buf->st_mtime == 0xffffffff)
	buf->st_mtime = 0;
      if (buf->st_ctime == 0xffffffff)
	buf->st_ctime = 0;
    }

  return ret;
}
