/*
 * $Id: utime.c,v 1.6 1995/11/08 15:21:55 miura Exp $
 *
 * utime.c -- set the access and modification times of file
 * Copyright (c) 1995, Shigeki Miura <miura@rpts.cl.nec.co.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <tchar.h>
#include <errno.h>
#include <time.h>
#include <sys/utime.h>


static char rcsid[] = "$Id: utime.c,v 1.6 1995/11/08 15:21:55 miura Exp $";


int
w32_utime (const _TCHAR *filename, struct _utimbuf *times)
{
  HANDLE hFile;
  SYSTEMTIME st;
  FILETIME ftLastAccess, ftLastWrite;
  DWORD dwAttributes;
  struct tm *tm;

  errno = 0;

  if (times)
    {
      tm = gmtime (&times->actime);
      st.wYear = (WORD)(tm->tm_year + 1900);
      st.wMonth = (WORD)(tm->tm_mon + 1);
      st.wDayOfWeek = (WORD) tm->tm_wday;
      st.wDay = (WORD) tm->tm_mday;
      st.wHour = (WORD) tm->tm_hour;
      st.wMinute = (WORD) tm->tm_min;
      st.wSecond = (WORD) tm->tm_sec;
      st.wMilliseconds = 0;
      SystemTimeToFileTime (&st, &ftLastAccess);

      tm = gmtime (&times->modtime);
      st.wYear = (WORD)(tm->tm_year + 1900);
      st.wMonth = (WORD)(tm->tm_mon + 1);
      st.wDayOfWeek = (WORD) tm->tm_wday;
      st.wDay = (WORD) tm->tm_mday;
      st.wHour = (WORD) tm->tm_hour;
      st.wMinute = (WORD) tm->tm_min;
      st.wSecond = (WORD) tm->tm_sec;
      st.wMilliseconds = 0;
      SystemTimeToFileTime (&st, &ftLastWrite);
    }
  else
    {
      GetSystemTime (&st);
      SystemTimeToFileTime (&st, &ftLastAccess);
      SystemTimeToFileTime (&st, &ftLastWrite);
    }

  dwAttributes = GetFileAttributes (filename);
  if (dwAttributes == 0xffffffff)
    {
      DWORD dwError = GetLastError ();

      if (dwError == ERROR_FILE_NOT_FOUND)
	errno = ENOENT;
      else
	errno = EACCES;
      return -1;
    }

  if (dwAttributes & FILE_ATTRIBUTE_READONLY)
    SetFileAttributes (filename, dwAttributes & ~FILE_ATTRIBUTE_READONLY);

  hFile = CreateFile (filename, GENERIC_WRITE, 0, NULL, OPEN_EXISTING,
		      FILE_ATTRIBUTE_NORMAL | FILE_FLAG_BACKUP_SEMANTICS,
		      NULL);
  if (hFile == INVALID_HANDLE_VALUE)
    {
      DWORD dwError = GetLastError ();

      if (dwError == ERROR_FILE_NOT_FOUND)
	errno = ENOENT;
      else if (dwError == ERROR_TOO_MANY_OPEN_FILES)
	errno = EMFILE;
      else
	errno = EACCES;
      SetFileAttributes (filename, dwAttributes);
      return -1;
    }

  if (!SetFileTime (hFile, NULL, &ftLastAccess, &ftLastWrite))
    {
      CloseHandle (hFile);
      SetFileAttributes (filename, dwAttributes);
      errno = EACCES;
      return -1;
    }

  CloseHandle (hFile);
  SetFileAttributes (filename, dwAttributes);

  return 0;
}
