/*
 * $Id: w32io.c,v 1.1 1995/11/08 17:46:54 miura Exp $
 *
 * w32io.c -- I/O routines for Windows NT
 * Copyright (c) 1995, Shigeki Miura <miura@rpts.cl.nec.co.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <winioctl.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>


static char rcsid[] = "$Id: w32io.c,v 1.1 1995/11/08 17:46:54 miura Exp $";


#if defined (_MSC_VER) && (_MSC_VER >= 900)
void __cdecl _dosmaperr (unsigned long); /* MSVCRTL internal function */
#endif /* _MSC_VER */

#define ARCHIVE_FILE	0
#define ARCHIVE_DRIVE	1
#define ARCHIVE_TAPE	2

static int fArchive = ARCHIVE_FILE;


static int
__w32_dev (const char *path)
{
  char buf[MAX_PATH];
  char *p;

  strcpy (buf, path);
  for (p = buf; *p != '\0'; p++)
    if (*p == '/')
      *p = '\\';

  if (!strncmp (buf, "\\\\.\\", 4))
    return 1;
  else
    return 0;
}


int
__w32_open (const char *path, int oflag)
{
  HANDLE hHandle;
  DWORD dwMode, dwCreate, dwCount;

  errno = 0;

  if (__w32_dev (path)
      && isalpha (path[4]) && path[5] == ':' && path[6] == '\0')
    fArchive = ARCHIVE_DRIVE;
  else
    fArchive = ARCHIVE_FILE;

  if (oflag & O_RDWR)
    dwMode = GENERIC_READ | GENERIC_WRITE;
  else if (oflag & O_WRONLY)
    dwMode = GENERIC_WRITE;
  else
    dwMode = GENERIC_READ;

  if (fArchive)
    dwCreate = OPEN_EXISTING;
  else if ((dwMode & GENERIC_READ) && (dwMode & GENERIC_WRITE))
    dwCreate = OPEN_ALWAYS;
  else if (dwMode & GENERIC_WRITE)
    dwCreate = CREATE_ALWAYS;
  else
    dwCreate = OPEN_EXISTING;

  hHandle = CreateFile (path, dwMode, 0, NULL, dwCreate,
			FILE_ATTRIBUTE_NORMAL, NULL);
  if (hHandle == INVALID_HANDLE_VALUE)
    {
      _dosmaperr (GetLastError ());
      return -1;
    }

  if (fArchive == ARCHIVE_DRIVE
      && !DeviceIoControl (hHandle, FSCTL_LOCK_VOLUME, NULL, 0, NULL, 0,
			   &dwCount, NULL))
    {
      _dosmaperr (GetLastError ());
      CloseHandle (hHandle);
      return -1;
    }

  return (int) hHandle;
}


int
__w32_creat (const char *path, int oflag)
{
  return __w32_open (path, O_CREAT | O_WRONLY);
}


int
__w32_close (int fd)
{
  DWORD dwCount;

  errno = 0;

  if (fArchive == ARCHIVE_DRIVE)
    DeviceIoControl ((HANDLE) fd, FSCTL_UNLOCK_VOLUME, NULL, 0, NULL, 0,
		     &dwCount, NULL);
  if (!CloseHandle ((HANDLE) fd))
    {
      _dosmaperr (GetLastError ());
      return -1;
    }

  return 0;
}


int
__w32_read (int fd, char *buf, unsigned int nbyte)
{
  DWORD dwCount, dwError;

  errno = 0;

  if (!ReadFile ((HANDLE) fd, buf, nbyte, &dwCount, NULL))
    {
      /*
       * tbs[EfBXNEhCu_hCuɃANZXĂ
       * ꍇAfBÃTCYzăANZXAZN^TCY
       * ̔{ȊÕoCgǂݍƂ ERROR_INVALID_PARAMETER
       * ̃G[ɂȂB
       *
       * ł ERROR_INVALID_PARAMETER ̃G[ɂȂꍇAfB
       * ÃTCYzăANZXƌȂ errno  0Aǂݍ
       * oCg 0 ԂB
       */
      dwError = GetLastError ();
      if (dwError == ERROR_INVALID_PARAMETER)
	{
	  errno = 0;
	  return 0;
	}
      else
	{
	  _dosmaperr (dwError);
	  return -1;
	}
    }

  return dwCount;
}


int
__w32_write (int fd, char *buf, unsigned int nbyte)
{
  DWORD dwCount, dwError;

  errno = 0;

  if (!WriteFile ((HANDLE) fd, buf, nbyte, &dwCount, NULL))
    {
      /*
       * tbs[EfBXNEhCu_hCuɃANZXĂ
       * ꍇAfBÃTCYzăANZXAZN^TCY
       * ̔{ȊÕoCgƂ ERROR_INVALID_PARAMETER
       * ̃G[ɂȂB
       *
       * ł ERROR_INVALID_PARAMETER ̃G[ɂȂꍇAfB
       * ÃTCYzăANZXƌȂ errno  ENOSPC ɂ
       *  -1 ԂB
       */
      dwError = GetLastError ();
      if (dwError == ERROR_INVALID_PARAMETER)
	errno = ENOSPC;
      else
	_dosmaperr (dwError);
      return -1;
    }

  return dwCount;
}


long
__w32_lseek (int fd, long offset, int whence)
{
  DWORD dwPos;

  errno = 0;

  dwPos = SetFilePointer ((HANDLE) fd, offset, NULL, whence);

  return dwPos;
}


void
__w32_ftruncate (int fd, long pos)
{
  SetFilePointer ((HANDLE) fd, pos, NULL, FILE_BEGIN);
  SetEndOfFile ((HANDLE) fd);
}


int
__w32_stdin (void)
{
  return (int) GetStdHandle (STD_INPUT_HANDLE);
}


int
__w32_stdout (void)
{
  return (int) GetStdHandle (STD_OUTPUT_HANDLE);
}
